package com.kexun.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.kexun.entity.ReportFileEntity;
import com.kexun.entity.ReportInfo;
import com.kexun.enums.FileTypeEnum;
import com.kexun.exception.RRException;
import com.kexun.service.ReportFileService;
import com.kexun.utils.DateUtils;
import com.kexun.utils.FormartUtils;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.xml.transform.*;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.*;
import java.util.Date;
import java.util.Map;

/**
 * Description: 文件处理实现类，包括文件的读取、写入
 *
 * @author lgs
 * @date 2023/12/22
 */
@Service
public class FileServiceImpl {

    @Autowired
    ReportFileService reportFileService;

    /**
     * 模型文件目标文件
     */
    @Value("${report.file.modelTargetPath}")
    private String modelTargetPath;
    /**
     * 解析出来的xml文件
     */
    @Value("${report.file.xmlTargetPath}")
    private String xmlTargetPath;
    /**
     * 存放JSON文件路径
     */
    @Value("${report.file.jsonTargetPath}")
    private String jsonTargetPath;

    /**
     * 上传文件
     *
     * @param context
     * @param filename
     * @param path
     */
    private void writeFile(String context, String filename, String path) {
        try {
            IOUtils.write(context, new FileOutputStream(path + filename), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 保存xml文件和json文件
     *
     * @param reportInfo
     * @param fileContext
     * @param fileType    文件类型
     * @param fileName
     * @param dateName
     * @param targetPath  目录路径
     */
    public void saveAndWriteFile(ReportInfo reportInfo, String fileContext, String fileType, String fileName, String dateName, String targetPath) {
        //根据HeadInfId查询是否存在
        ReportFileEntity reportFileEntity = new ReportFileEntity();
        if (reportInfo.getHeadInfId() != null) {
            QueryWrapper<ReportFileEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("head_inf_id", reportInfo.getHeadInfId());
            queryWrapper.eq("file_type", fileType);
            ReportFileEntity reportFileEntityResult = reportFileService.getOne(queryWrapper);
            if (reportFileEntityResult != null) {
                reportFileEntity.setId(reportFileEntityResult.getId());
            }
        }
        reportFileEntity.setHeadInfId(reportInfo.getHeadInfId());
        reportFileEntity.setRptNo(reportInfo.getRptNo());
        reportFileEntity.setBusinessId(reportInfo.getBusinessId());
        reportFileEntity.setBusinessNum(reportInfo.getBusinessNum());
        reportFileEntity.setFileOnOff("1");
        reportFileEntity.setFileType(fileType);
        reportFileEntity.setFileTypeDesc(FileTypeEnum.findEnumByCode(fileType).getName());
        reportFileEntity.setFileName(fileName);
        reportFileEntity.setFilePath(dateName + File.separator + fileName);
        //保存文件信息表
        reportFileService.saveOrUpdate(reportFileEntity);
        //上传文件到服务器的目录
        //xml文件
        String filePath = targetPath + File.separator + dateName + File.separator;
        File file = new File(filePath);
        if (!file.exists()) {
            file.mkdirs();
        }
        writeFile(fileContext, fileName, filePath);
    }

    /**
     * 文件上传保存
     *
     * @param multipartFile
     * @param headInfId
     * @param fileType
     */
    public void saveAndUploadFile(MultipartFile multipartFile, Long headInfId, String fileType,String businessNum) {
        //查询
        QueryWrapper<ReportFileEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("head_inf_id", headInfId);
        queryWrapper.eq("file_type", fileType);
        ReportFileEntity reportFileEntityResult = reportFileService.getOne(queryWrapper);
        if (reportFileEntityResult == null) {
            reportFileEntityResult = new ReportFileEntity();
            reportFileEntityResult.setHeadInfId(headInfId);
            reportFileEntityResult.setBusinessNum(businessNum);
            reportFileEntityResult.setFileOnOff("1");
        }
        String originalFilename = multipartFile.getOriginalFilename();
        //上传文件
        String date = DateUtils.format(new Date(), DateUtils.YYYYMMDD);
        String filePath = "";
        //模型文件
        reportFileEntityResult.setFileName(originalFilename);
        reportFileEntityResult.setFilePath(date + File.separator + originalFilename);
        //解析后xml文件
        if (fileType.equals(FileTypeEnum.XML.getCode())) {
            filePath = xmlTargetPath + date;
            reportFileEntityResult.setFileType(FileTypeEnum.XML.getCode());
            reportFileEntityResult.setFileTypeDesc(FileTypeEnum.XML.getName());
            //解析后json文件
        } else if (fileType.equals(FileTypeEnum.JSON.getCode())) {
            filePath = jsonTargetPath + date;
            reportFileEntityResult.setFileType(FileTypeEnum.JSON.getCode());
            reportFileEntityResult.setFileTypeDesc(FileTypeEnum.JSON.getName());
        } else if (fileType.equals(FileTypeEnum.MODEL.getCode())) {
            filePath = modelTargetPath + date;
            reportFileEntityResult.setFileType(FileTypeEnum.MODEL.getCode());
            reportFileEntityResult.setFileTypeDesc(FileTypeEnum.MODEL.getName());
        }
        File file = new File(filePath);
        if (!file.exists()) {
            file.mkdirs();
        }
        try {
            InputStream inputStream = multipartFile.getInputStream();
            FileOutputStream outputStream = new FileOutputStream(new File(filePath, originalFilename));
            IOUtils.copy(inputStream, outputStream);
            inputStream.close();
            outputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        //更新目录
        reportFileService.saveOrUpdate(reportFileEntityResult);
    }
}
