package com.kexun.utils;

import com.kexun.utils.compare.HistoryDesc;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.util.LinkedList;
import java.util.List;

/**
 * Description:对象比较工具
 *
 * @author lgs
 * @date 2023/05/17
 */
@Slf4j
public class CompareObjectUtils {

    private static final String GETTER_PREFIX = "get";



    /**
     * 对比两个对象同名变量（或者扩展名称）名称，对比内容以字符串呈现
     *
     * @param newObjectWithAnnotation
     *            含有@ApiModelProperty或者@ObjectVariable注解的新对象，对比出现不相同时，需要获取到该变量的中文名称
     * @param oldObject
     *            旧对象
     * @return 对比内容以字符串呈现
     */
    public static String compareTwoObjectForDesc(Object newObjectWithAnnotation, Object oldObject) {
        List<HistoryDesc> list = compareTwoObject(newObjectWithAnnotation, oldObject);
        if (list != null && list.size() > 0) {
            return list.toString();
        }
        return "";
    }

    /**
     * 对比两个对象同名变量（或者扩展名称）名称，对比内容以map呈现
     *
     * @param newObjectWithAnnotation
     *            含有@ApiModelProperty或者@ObjectVariable注解的新对象，对比出现不相同时，需要获取到该变量的中文名称
     * @param oldObject
     *            旧对象
     * @return 对比内容以List呈现(update by RongTongXin 2019-05-16)
     */
    public static List<HistoryDesc> compareTwoObject(Object newObjectWithAnnotation, Object oldObject) {
        List<HistoryDesc> list = new LinkedList<>();
        loopForSuperClass(list, newObjectWithAnnotation, oldObject, newObjectWithAnnotation.getClass());
        return list;
    }
    /**
     * 遍历对象所有的继承类
     *
     * @param list
     *            内容不相同的list(update By RongTongxin 2019-05-16 由Map改为使用List<HistoryDesc>的方式,解决日期等序列化问题)
     * @param newObjectWithAnnotation
     *            含有@ApiModelProperty或者@ObjectVariable注解的新对象，对比出现不相同时，需要获取到该变量的中文名称
     * @param oldObject
     *            旧对象
     * @param newClazz
     *            新对象Class
     */
    private static void loopForSuperClass(List<HistoryDesc> list, Object newObjectWithAnnotation, Object oldObject, Class<?> newClazz) {
        // 获取对象的class
        // Class<?> newClazz = newObjectWithAnnotation.getClass();
        Class<?> oldClazz = oldObject.getClass();
        // 获取对象的属性数组
        Field[] newFields = newClazz.getDeclaredFields();
        Field[] oldFields = oldClazz.getDeclaredFields();
        // 遍历newFields
        if (newFields != null && newFields.length > 0) {
            for (int i = 0; i < newFields.length; i++) {
                ApiModelProperty apiModelProperty = newFields[i].getAnnotation(ApiModelProperty.class);
                if (apiModelProperty != null) {
                    // 直接通过该变量名获取值对比
                    Object value1 = invokeGetter(newObjectWithAnnotation, newFields[i].getName());
                    Object value2 = invokeGetter(oldObject, newFields[i].getName());
                    if (!compareTwoVal(value1, value2)) {
                        // 不相同
                        HistoryDesc historyDesc = new HistoryDesc(newFields[i].getName(),
                                StringUtils.isNotBlank(apiModelProperty.value()) ? apiModelProperty.value() : newFields[i].getName(), value2, value1);
                        list.add(historyDesc);
                    }
                } else {
                    // 无对象变量注解，不处理
                }
            }
        }
        // 往父类找
        if (newClazz.getSuperclass() != Object.class) {
            loopForSuperClass(list, newObjectWithAnnotation, oldObject, newClazz.getSuperclass());
        }
    }


    public static <E> E invokeGetter(Object obj, String propertyName) {
        Object object = obj;
        for (String name : StringUtils.split(propertyName, ".")) {
            String getterMethodName = GETTER_PREFIX + StringUtils.capitalize(name);
            object = invokeMethod(object, getterMethodName, new Class[]{}, new Object[]{});
        }
        return (E) object;
    }

    /**
     * 对比两个数据是否内容相同
     *
     * @param object1,object2
     * @return boolean类型
     */
    public static boolean compareTwoVal(Object object1, Object object2) {
        if (object1 == null && object2 == null) {
            return true;
        }
        if (object1 == null && object2 != null) {
            return false;
        }
        if (StringUtils.equals(ObjectUtils.toString(object1), ObjectUtils.toString(object2))) {
            return true;
        }
        //比较的类型如果是浮点数类的话
        if ((object1 instanceof BigDecimal || object1 instanceof Double || object1 instanceof Float)
                && (object2 instanceof BigDecimal || object2 instanceof Double || object2 instanceof Float)
        ) {
            BigDecimal bigDecimal1 = new BigDecimal(object1.toString());
            BigDecimal bigDecimal2 = new BigDecimal(object2.toString());
            if (bigDecimal1.compareTo(bigDecimal2) == 0) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }

    public static <E> E invokeMethod(final Object obj, final String methodName, final Class<?>[] parameterTypes,
                                     final Object[] args) {
        if (obj == null || methodName == null) {
            return null;
        }
        Method method = getAccessibleMethod(obj, methodName, parameterTypes);
        if (method == null) {
            //throw new IllegalArgumentException("在 [" + obj.getClass() + "] 中，没有找到 [" + methodName + "] 方法 ");
            log.debug("在 [" + obj.getClass() + "] 中，没有找到 [" + methodName + "] 方法 ");
            return null;
        }
        try {
            return (E) method.invoke(obj, args);
        } catch (Exception e) {
            String msg = "method: " + method + ", obj: " + obj + ", args: " + args + "";
            throw new RuntimeException(msg, e);
        }
    }

    /**
     * 循环向上转型, 获取对象的DeclaredMethod,并强制设置为可访问.
     * 如向上转型到Object仍无法找到, 返回null.
     * 匹配函数名+参数类型。
     * 用于方法需要被多次调用的情况. 先使用本函数先取得Method,然后调用Method.invoke(Object obj, Object... args)
     */
    public static Method getAccessibleMethod(final Object obj, final String methodName,
                                             final Class<?>... parameterTypes) {
        // 为空不报错。直接返回 null
        // Validate.notNull(obj, "object can't be null");
        if (obj == null) {
            return null;
        }
        Validate.notNull(methodName, "methodName can't be blank");
        for (Class<?> searchType = obj.getClass(); searchType != Object.class; searchType = searchType.getSuperclass()) {
            try {
                Method method = searchType.getDeclaredMethod(methodName, parameterTypes);
                makeAccessible(method);
                return method;
            } catch (NoSuchMethodException e) {
                // Method不在当前类定义,继续向上转型
                continue;// new add
            }
        }
        return null;
    }

    public static void makeAccessible(Method method) {
        if ((!Modifier.isPublic(method.getModifiers()) || !Modifier.isPublic(method.getDeclaringClass().getModifiers()))
                && !method.isAccessible()) {
            method.setAccessible(true);
        }
    }
}
